define([
    'jquery',
    'underscore',
    'codemirror/codemirror',
    'uiRegistry'
], function ($, _, CodeMirror, registry) {
    'use strict';

    /**
     * Adds selectRange method to jQuery.
     *
     * This method changes cursor position in the textarea if 3rd parameter is not passed
     * and choose selection of a text if 3rd parameter exist.
     *
     * @param line  int - line number
     * @param start int - start number of selection
     * @param end   int - end number of selection
     */
    $.fn.selectRange = function(line, start, end) {
        //console.log(line);
        //console.log(start);
        // change cursor start position according to line number
        if (line > 0) {
            // add number of new line symbols
            start += line;
            // ignore current line
            line--;
            var content = this.val().split('\n');
            //console.log(content);
            for (; line >= 0; line--) {
                //console.log(content[line].length);
                start += content[line].length;
            }
        }
        //console.log(start);

        if(end === undefined) {
            end = start;
        }

        return this.each(function() {
            if('selectionStart' in this) {
                this.selectionStart = start;
                this.selectionEnd = end;
            } else if(this.setSelectionRange) {
                this.setSelectionRange(start, end);
            } else if(this.createTextRange) {
                var range = this.createTextRange();
                range.collapse(true);
                range.moveEnd('character', end);
                range.moveStart('character', start);
                range.select();
            }
        });
    };

    $.widget('mirasvit.editor', {
        options: {
            name: null
        },

        _create: function () {
            var self = this;

            self.editors = {};
            self.autoRefresh = false;

            self.$previewFrame = $('[data-role=preview]');
            self.$preview = $(self.$previewFrame[0].contentDocument || self.$previewFrame[0].contentWindow.document);
            self.$desktopView = $('[data-role=desktop-view]');
            self.$mobileView = $('[data-role=mobile-view]');
            self.$autoRefresh = $('[data-role=auto-refresh]');
            self.$refresh = $('[data-role=refresh]');
            self.$spinner = $('.email-designer__editor-preview .spinner[data-role=spinner]');

            _.bindAll(this,
                'fitFrame',
                'updatePreview',
                'changeView'
            );

            self.$desktopView.on('click', function (e) {
                e.preventDefault();
                self.changeView('desktop');
            });

            self.$mobileView.on('click', function (e) {
                e.preventDefault();
                self.changeView('mobile');
            });

            self.$autoRefresh.on('change', function (e) {
                self.$refresh.parent().toggle();
                self.autoRefresh = self.$autoRefresh.attr('checked');
            });

            self.$refresh.on('click', function (e) {
                e.preventDefault();
                self.updatePreview();
            });

            $('.editor').each(function (index, item) {
                var editor = CodeMirror.fromTextArea(item, {
                    mode: 'htmlmixed',
                    matchTags: true,
                    viewportMargin: Infinity,
                    styleActiveLine: true,
                    tabSize: 2,
                    lineWrapping: false,
                    lineNumbers: true,
                    tabMode: 'indent'
                });

                var delay = 0;

                editor.on('change', function (instance, changeObj) {
                    // update editor and element values
                    editor.refresh();
                    editor.save();

                    var area = $(editor.getTextArea());

                    // update value in UI component for successful save on server side
                    var component = registry.get({inputName: area.attr('name')});
                    component.value(area.val());

                    if (self.autoRefresh) {
                        clearTimeout(delay);
                        delay = setTimeout(self.updatePreview, 300);
                    }
                });

                // change cursor position when changing data in the editor
                editor.doc.on('cursorActivity', function(doc) {
                    $(editor.getTextArea()).selectRange(
                        doc.sel.ranges[0].anchor.line,
                        doc.sel.ranges[0].anchor.ch
                    );
                });

                // update cursor position in a textarea
                editor.on('beforeSelectionChange', function(doc, selection) {
                    $(editor.getTextArea()).selectRange(
                        selection.ranges[0].anchor.line,
                        selection.ranges[0].anchor.ch
                    );
                });

                // update editor values on changing textarea
                $(item).on('change', function() {
                    editor.setValue(this.value);
                    editor.refresh();
                    editor.save();
                });

                self.editors[$(item).attr('name')] = editor;
            });

            setInterval(function () {
                self.fitFrame();
            }, 500);

            self.updatePreview();
        },

        fitFrame: function () {
            var self = this;

            var areasHeight = $('.email-designer__editor-areas').height();
            var previewHeight = $('body', self.$preview).height();

            var max = areasHeight;
            if (previewHeight > max) {
                max = previewHeight;
            }

            self.$previewFrame.height(max);
        },

        updatePreview: function () {
            var self = this;
            var data = {};

            self.$previewFrame.css('opacity', '0.5');
            self.$spinner.show();

            _.each(self.editors, function (editor, key) {
                editor.refresh();
                editor.save();

                var $textarea = $(editor.getTextArea());
                data[$textarea.data('area')] = $textarea.val();
            });

            self.abortAllAjax();

            var jqXHR = $.ajax(self.options.url, {
                type: 'POST',
                data: data,

                complete: function (response) {
                    self.$preview[0].open();
                    self.$preview[0].write(response.responseText);
                    self.$preview[0].close();

                    self.fitFrame();

                    self.$previewFrame.css('opacity', '1');
                    self.$spinner.hide();
                }
            });

            window.xhrPool.push(jqXHR);
        },

        changeView: function (mode) {
            var self = this;

            if (mode == 'desktop') {
                self.$desktopView.removeClass('tertiary').addClass('secondary');
                self.$mobileView.removeClass('primary').addClass('tertiary');
                self.$previewFrame.width('100%');
            } else {
                self.$mobileView.removeClass('tertiary').addClass('secondary');
                self.$desktopView.removeClass('primary').addClass('tertiary');
                self.$previewFrame.width('300px');
            }

            self.fitFrame();
        },

        abortAllAjax: function () {
            $(window.xhrPool).each(function (idx, jqXHR) {
                jqXHR.abort();
            });
            window.xhrPool = [];
        }
    });

    window.xhrPool = [];

    $.ajaxSetup({
        complete: function (jqXHR) {
            var index = window.xhrPool.indexOf(jqXHR);
            if (index > -1) {
                window.xhrPool.splice(index, 1);
            }
        }
    });

    return $.mirasvit.editor;
});